var assert = require('assert');
var RSA = require('node-rsa');

function asPrivateKey(privateKey) {
  if (typeof(privateKey) === 'string') {
    var key = new RSA();

    key.import(privateKey, 'pkcs1-private-pem');

    return key;
  }

  return privateKey;
}

function asPublicKey(publicKey) {
  if (typeof(publicKey) === 'string') {
    var key = new RSA();

    key.import(publicKey, 'pkcs1-public-pem');

    return key;
  }

  return publicKey;
}

var utils = {
  asPublicKey: asPublicKey,
  asPrivateKey: asPrivateKey,
  encryptAndSign: function (input, signKey, encryptKey) {
    signKey = asPrivateKey(signKey);
    encryptKey = asPublicKey(encryptKey);

    return utils.sign(utils.encrypt(input, encryptKey), signKey);
  },
  decryptAndVerify: function (input, decryptKey, verifyKey) {
    verifyKey = asPrivateKey(verifyKey);
    decryptKey = asPublicKey(decryptKey);

    var verifies = utils.verify(input, verifyKey);

    assert.equal(true, verifies, 'does not verify signature');

    return utils.decrypt(input.data, decryptKey);
  },
  encrypt: function (input, publicKey) {
    publicKey = asPublicKey(publicKey);

    var normalInput = JSON.stringify(input);

    var buffer = publicKey.encrypt(normalInput);

    return buffer.toString('base64');
  },
  decrypt: function (input, privateKey) {
    privateKey = asPrivateKey(privateKey);

    var buffer = new Buffer(input, 'base64');

    var decrypted = privateKey.decrypt(input);

    return JSON.parse(decrypted);
  },
  sign: function (input, privateKey) {
    privateKey = asPrivateKey(privateKey);

    var normalInput = JSON.stringify(input);

    return {
      data: normalInput,
      signature: privateKey.sign(normalInput, 'base64')
    };
  },
  verify: function (input, publicKey) {
    publicKey = asPrivateKey(publicKey);

    return publicKey.verify(input.data, input.signature, null, 'base64');
  }
};

module.exports = utils;
